using System;
using System.Reflection;
using System.Runtime.CompilerServices;

// Information about this assembly is defined by the following attributes. 
// Change them to the values specific to your project.

[assembly: AssemblyTitle ("Mono.Upnp")]
[assembly: AssemblyDescription ("A client and server Universal Plug 'n Play stack.")]
[assembly: AssemblyConfiguration ("")]
[assembly: AssemblyCompany ("")]
[assembly: AssemblyProduct ("")]
[assembly: AssemblyCopyright ("")]
[assembly: AssemblyTrademark ("")]
[assembly: AssemblyCulture ("")]
[assembly: CLSCompliant (true)]

// The assembly version has the format "{Major}.{Minor}.{Build}.{Revision}".
// The form "{Major}.{Minor}.*" will automatically update the build and revision,
// and "{Major}.{Minor}.{Build}.*" will update just the revision.

[assembly: AssemblyVersion ("1.0.*")]

// The following attributes are used to specify the signing key for the assembly, 
// if desired. See the Mono documentation for more information about signing.

[assembly: AssemblyDelaySign (false)]
[assembly: InternalsVisibleTo ("Mono.Upnp.Dcp.MediaServer1, PublicKey=" +
                               "00240000048000009400000006020000002400" +
                               "0052534131000400001100000007CA7138D878" +
                               "FFB4E68E1222C8FD84557E0336F4FB76D83774" +
                               "13CC48DAB29CE14BEA65AAB4250A04FB059F39" +
                               "CCE2593DC29ADAF8954E922F4A6354B6DEB71E" +
                               "909137B33DEA212BB04F1492FBDFE50B0B7614" +
                               "2974F2E5D86049D7F44706839CD76C57FAC02E" +
                               "86C977188DD5185F5DDBA8EE46CA8F6F40224F" +
                               "9D8653092BBAFE9F")]

[assembly: InternalsVisibleTo ("Mono.Upnp.Dcp.MediaServer1.FileSystem, PublicKey=" +
                               "0024000004800000940000000602000000240000525341310" +
                               "0040000110000007FD08FD4F03157DEE6573823B967BCF78D" +
                               "A200777DF681030D1C8EC06BEA3BE2C9CEC6735CF0046953E" +
                               "1949882B0FB1D6F13ED43A72465BDA6EB3DA794D98D3921AF" +
                               "D24996E2FFE2FCF87B2B715DA8F20363314259C8F280E71C7" +
                               "46781846E73D54C383DC32B8C19038F317F9394815C0307BE" +
                               "92D0BA9CB278C2019B31A28D83")]
