// 
// ClassReference.cs
//  
// Author:
//       Scott Peterson <lunchtimemama@gmail.com>
// 
// Copyright (c) 2009 Scott Peterson
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Mono.Upnp.Xml;

namespace Mono.Upnp.Dcp.MediaServer1.ContentDirectory1
{
    public class ClassReference : Class, IComparable<ClassReference>, IEquatable<ClassReference>
    {
        protected ClassReference ()
        {
        }
        
        public ClassReference (bool includeDerived, string fullClassName)
            : this (includeDerived, fullClassName, null)
        {
        }

        public ClassReference (bool includeDerived, string fullClassName, string friendlyClassName)
            : base (fullClassName, friendlyClassName)
        {
            IncludeDerived = includeDerived;
        }

        [XmlAttribute ("includeDerived")]
        public virtual bool IncludeDerived { get; protected set; }
        
        int IComparable<ClassReference>.CompareTo (ClassReference classReference)
        {
            return FullClassName.CompareTo (classReference.FullClassName);
        }

        public override bool Equals (object obj)
        {
            return Equals (obj as ClassReference);
        }

        public bool Equals (ClassReference classReference)
        {
            return base.Equals (classReference) && classReference.IncludeDerived == IncludeDerived;
        }

        public override int GetHashCode ()
        {
            var hashCode = base.GetHashCode ();
            if (IncludeDerived) {
                return hashCode;
            } else {
                return ~hashCode;
            }
        }
    }
}
