
// This file has been generated by the GUI designer. Do not modify.
namespace Mono.Upnp.GtkClient
{
	public partial class MainWindow
	{
		private global::Gtk.UIManager UIManager;
		private global::Gtk.VBox vbox1;
		private global::Gtk.HPaned hpaned;
		private global::Gtk.ScrolledWindow GtkScrolledWindow;
		private global::Gtk.TreeView list;
		private global::Gtk.VBox infoBox;
		private global::Gtk.Label infoFiller;
        
		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget Mono.Upnp.GtkClient.MainWindow
			this.UIManager = new global::Gtk.UIManager ();
			global::Gtk.ActionGroup w1 = new global::Gtk.ActionGroup ("Default");
			this.UIManager.InsertActionGroup (w1, 0);
			this.AddAccelGroup (this.UIManager.AccelGroup);
			this.WidthRequest = 0;
			this.HeightRequest = 0;
			this.Name = "Mono.Upnp.GtkClient.MainWindow";
			this.Title = global::Mono.Unix.Catalog.GetString ("Mono.Upnp");
			this.WindowPosition = ((global::Gtk.WindowPosition)(1));
			// Container child Mono.Upnp.GtkClient.MainWindow.Gtk.Container+ContainerChild
			this.vbox1 = new global::Gtk.VBox ();
			this.vbox1.Name = "vbox1";
			this.vbox1.Spacing = 6;
			// Container child vbox1.Gtk.Box+BoxChild
			this.hpaned = new global::Gtk.HPaned ();
			this.hpaned.CanFocus = true;
			this.hpaned.Name = "hpaned";
			this.hpaned.Position = 354;
			this.hpaned.BorderWidth = ((uint)(5));
			// Container child hpaned.Gtk.Paned+PanedChild
			this.GtkScrolledWindow = new global::Gtk.ScrolledWindow ();
			this.GtkScrolledWindow.Name = "GtkScrolledWindow";
			this.GtkScrolledWindow.ShadowType = ((global::Gtk.ShadowType)(1));
			// Container child GtkScrolledWindow.Gtk.Container+ContainerChild
			this.list = new global::Gtk.TreeView ();
			this.list.CanFocus = true;
			this.list.Name = "list";
			this.GtkScrolledWindow.Add (this.list);
			this.hpaned.Add (this.GtkScrolledWindow);
			global::Gtk.Paned.PanedChild w3 = ((global::Gtk.Paned.PanedChild)(this.hpaned [this.GtkScrolledWindow]));
			w3.Resize = false;
			// Container child hpaned.Gtk.Paned+PanedChild
			this.infoBox = new global::Gtk.VBox ();
			this.infoBox.Name = "infoBox";
			this.infoBox.Spacing = 6;
			// Container child infoBox.Gtk.Box+BoxChild
			this.infoFiller = new global::Gtk.Label ();
			this.infoFiller.Name = "infoFiller";
			this.infoFiller.LabelProp = global::Mono.Unix.Catalog.GetString ("Select a device or service");
			this.infoBox.Add (this.infoFiller);
			global::Gtk.Box.BoxChild w4 = ((global::Gtk.Box.BoxChild)(this.infoBox [this.infoFiller]));
			w4.Position = 0;
			w4.Fill = false;
			this.hpaned.Add (this.infoBox);
			this.vbox1.Add (this.hpaned);
			global::Gtk.Box.BoxChild w6 = ((global::Gtk.Box.BoxChild)(this.vbox1 [this.hpaned]));
			w6.Position = 0;
			this.Add (this.vbox1);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.DefaultWidth = 1077;
			this.DefaultHeight = 527;
			this.Show ();
			this.DeleteEvent += new global::Gtk.DeleteEventHandler (this.OnDeleteEvent);
		}
	}
}
