// 
// Reference.cs
//  
// Author:
//       Scott Thomas <lunchtimemama@gmail.com>
// 
// Copyright (c) 2010 Scott Thomas
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Mono.Upnp.Dcp.MediaServer1.Xml;
using Mono.Upnp.Xml;

namespace Mono.Upnp.Dcp.MediaServer1.ContentDirectory1
{
    public class Reference<T> : IXmlSerializable<VirtualContext>
        where T : Item
    {
        T item;
        string id;
        string parent_id;

        public Reference (string id, string parentId, T item)
        {
            if (id == null) {
                throw new ArgumentNullException ("id");
            } else if (parentId == null) {
                throw new ArgumentNullException ("parentId");
            } else if (item == null) {
                throw new ArgumentNullException ("item");
            }

            this.id = id;
            this.parent_id = parentId;
            this.item = item;
        }

        public void Serialize (XmlSerializationContext<VirtualContext> context)
        {
            if (context == null) {
                throw new ArgumentNullException ("context");
            }

            context.AutoSerializeObjectStart (item);
            SerializeMembers (context);
            context.AutoSerializeObjectEnd (item);
        }

        public void SerializeMembers (XmlSerializationContext<VirtualContext> context)
        {
            if (context == null) {
                throw new ArgumentNullException ("context");
            }

            context.AutoSerializeMembers (item, new VirtualContext (
                new Override ("id", Schemas.DidlLiteSchema, id),
                new Override ("parentID", Schemas.DidlLiteSchema, parent_id),
                new Override ("refID", Schemas.DidlLiteSchema, item.Id)
            ));
        }
    }
}
