
// This file has been generated by the GUI designer. Do not modify.
namespace Mono.Upnp.GtkClient
{
	public partial class ActionInvocationWindow
	{
		private global::Gtk.VBox vbox;
		private global::Gtk.Label name;
		private global::Gtk.VBox inputsBox;
		private global::Gtk.Button invoke;
		private global::Gtk.VBox outputsBox;
        
		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget Mono.Upnp.GtkClient.ActionInvocationWindow
			this.WidthRequest = 0;
			this.HeightRequest = 0;
			this.Name = "Mono.Upnp.GtkClient.ActionInvocationWindow";
			this.Title = "Invoke Action";
			this.WindowPosition = ((global::Gtk.WindowPosition)(1));
			this.DefaultWidth = 0;
			this.DefaultHeight = 0;
			// Container child Mono.Upnp.GtkClient.ActionInvocationWindow.Gtk.Container+ContainerChild
			this.vbox = new global::Gtk.VBox ();
			this.vbox.Name = "vbox";
			this.vbox.Spacing = 6;
			this.vbox.BorderWidth = ((uint)(6));
			// Container child vbox.Gtk.Box+BoxChild
			this.name = new global::Gtk.Label ();
			this.name.Name = "name";
			this.name.Xalign = 0F;
			this.name.LabelProp = "<big><b>action</b></big>";
			this.name.UseMarkup = true;
			this.vbox.Add (this.name);
			global::Gtk.Box.BoxChild w1 = ((global::Gtk.Box.BoxChild)(this.vbox [this.name]));
			w1.Position = 0;
			w1.Expand = false;
			w1.Fill = false;
			// Container child vbox.Gtk.Box+BoxChild
			this.inputsBox = new global::Gtk.VBox ();
			this.inputsBox.Name = "inputsBox";
			this.inputsBox.Spacing = 6;
			this.vbox.Add (this.inputsBox);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(this.vbox [this.inputsBox]));
			w2.Position = 1;
			// Container child vbox.Gtk.Box+BoxChild
			this.invoke = new global::Gtk.Button ();
			this.invoke.CanFocus = true;
			this.invoke.Name = "invoke";
			this.invoke.UseUnderline = true;
			this.invoke.Relief = ((global::Gtk.ReliefStyle)(2));
			// Container child invoke.Gtk.Container+ContainerChild
			global::Gtk.Alignment w3 = new global::Gtk.Alignment (0.5F, 0.5F, 0F, 0F);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w4 = new global::Gtk.HBox ();
			w4.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w5 = new global::Gtk.Image ();
			w5.Pixbuf = global::Stetic.IconLoader.LoadIcon (this, "gtk-execute", global::Gtk.IconSize.Menu);
			w4.Add (w5);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w7 = new global::Gtk.Label ();
			w7.LabelProp = global::Mono.Unix.Catalog.GetString ("_Invoke");
			w7.UseUnderline = true;
			w4.Add (w7);
			w3.Add (w4);
			this.invoke.Add (w3);
			this.vbox.Add (this.invoke);
			global::Gtk.Box.BoxChild w11 = ((global::Gtk.Box.BoxChild)(this.vbox [this.invoke]));
			w11.Position = 2;
			w11.Expand = false;
			w11.Fill = false;
			// Container child vbox.Gtk.Box+BoxChild
			this.outputsBox = new global::Gtk.VBox ();
			this.outputsBox.Name = "outputsBox";
			this.outputsBox.Spacing = 6;
			this.vbox.Add (this.outputsBox);
			global::Gtk.Box.BoxChild w12 = ((global::Gtk.Box.BoxChild)(this.vbox [this.outputsBox]));
			w12.Position = 3;
			this.Add (this.vbox);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Show ();
			this.invoke.Clicked += new global::System.EventHandler (this.OnInvokeClicked);
		}
	}
}
